/*                                                                
**  Copyright (C) 2001,2007,2011  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* H****************************************************************** 
 * 
 * FILE NAME: dsnan.h
 *
 * DEVELOPMENT: libdev
 * 
 * DESCRIPTION: The macros defined in this header file can be used
 *              to test whether a number is normal  or NaN or INF.
 *              Also, macros are defined to set NaN values for floats
 *              and doubles.
 * LAST MODIFIED: 
H********************************************************************/

#ifndef DSNAN_H
#define DSNAN_H

#ifdef __cplusplus
extern "C" {
#endif



/* 32 bit floating NaN */
#define FNANVAL 0x7FF80000  /* should be 0x7FC0000 */

/* top 32 bits of double NaN */
#define DNANVAL 0x7FF80000
 
/* 32 bit mask to catch floating NaN or Inf */
#define FNANMASK        0x7F800000

/* 32 bit mask to catch double NaN or Inf */
#define DNANMASK        0x7FF00000



#if defined(__alpha) && defined(__unix__)
/************************************************************
  Alpha / Unix definitions
************************************************************/

#include <nan.h>
#include <math.h>

/* Extract value of float and cast to unsigned int (32 bits) */
#define FNANCAST(M)     ((unsigned int )(*(unsigned int*)&(M)) )

/* test for floating NaN */
#define ds_fNAN(L) \
((float)(*((unsigned int*) (&(L)))) == 0.0 ?  0 : \
( ( FNANCAST(L) & FNANMASK) == FNANMASK ?  1 : 0))
/* test for double NaN */
#define ds_dNAN(X) (!finite((X)))

/* Set floating or double NaN values */
#define ds_MAKE_FNAN(x) (*(unsigned int *)&(x)=FNANVAL)
#define ds_MAKE_DNAN(X)  (((dnan *)&(X))->nan_parts.exponent = 0x7ff) ; \
 (((dnan *)&(X))->nan_parts.qnan_bit = 0x1)

#endif



/************************************************************
  ENDIAN definitions
************************************************************/

#if defined(__APPLE__)
#include <machine/endian.h>
#elif defined(__linux__)
#include <endian.h>
#elif defined(__sparc) || defined(sparc)
#ifndef BIG_ENDIAN
#define BIG_ENDIAN 1
#endif
#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN 0
#endif
#define BYTE_ORDER BIG_ENDIAN
#elif defined(__CYGWIN32__)
#define BYTE_ORDER LITTLE_ENDIAN
#endif



/* Manufacture pointer to floating NaN */
#define FNANPTR(x) ((int*)((void*)&(x)))

#if BYTE_ORDER == LITTLE_ENDIAN
/* Manufacture pointers to high and low words of double NaN */
#define DNANPTR1(x) (((int *)((void*)(&(x))))+1)
#define DNANPTR2(x)  ((int *)((void*)(&(x))))
#else
/* Manufacture pointers to high and low words of double NaN */
#define DNANPTR1(x)  ((int *)((void*)(&(x))))
#define DNANPTR2(x) (((int *)((void*)(&(x))))+1)
#endif





/* Test for floating NaN */
#define ds_fNAN(L) (((L)==(L))?0:1) 
 
/* Test for double NaN */
#define ds_dNAN(L) (((L)==(L))?0:1)



/* Set floating or double NaN values */



#define ds_MAKE_FNAN(x) {int __DS_NAN = FNANVAL; \
   memcpy(FNANPTR((x)),&__DS_NAN, sizeof(int));}
#define ds_MAKE_DNAN(x) {int __DS_NAN = DNANVAL; \
   memcpy(DNANPTR1((x)),&__DS_NAN, sizeof(int)); \
   memset(DNANPTR2((x)),0, sizeof(int)); }


#ifdef __cplusplus
  }
#endif

#endif /* DSNAN_H */
    
