#! /bin/sh

TESTID=prism-smoke001
export TESTID

#
# Replace with variable via configure
#
ASCDS_QUIET=1; export ASCDS_QUIET
ASCDS_OVERRIDE=1; export ASCDS_OVERRIDE
. setup.sh

#
# Setup test env
#
TESTDIR=$ASCDS_INSTALL/test/smoke/data; export TESTDIR
if [ "$ASCDS_SMOKE_DIR" = "" ]; then
   OUTDIR=$ASCDS_WORK_PATH/smoke.${LOGNAME}/$TESTID; export OUTDIR
else
   OUTDIR=$ASCDS_SMOKE_DIR/smoke.${LOGNAME}/$TESTID; export OUTDIR
fi
SAVEDIR=$TESTDIR; export SAVEDIR

INDIR=$TESTDIR

OUTFILE=${OUTDIR}/${TESTID}.txt
OUTFILE2=${OUTDIR}/${TESTID}_b.txt


if test -d $OUTDIR
then
   \rm -rf $OUTDIR
fi
mkdir -p $OUTDIR

#
# Run test
#

printf "Running test $TESTID "
echo " PRISM2 SMOKE TEST LOG " > $OUTFILE
echo "" >> $OUTFILE

prism &
printf "."
X=0 
sleep 5
# wait upto five minutes before exiting
while [ $X -le 300 ]
do
   a=`xpaaccess prism` 
   if  [ "$a" = "yes" ] ; then
      X=500
   else 
      sleep 1 
      X=`expr $X + 1`
   fi
done

printf "."
if [ $X -eq 500 ] ; then
   echo " -- TEST LOAD ASCII TABLE" >>  $OUTFILE 
   xpaset -p prism file $INDIR/rgb.txt >> $OUTFILE
   sleep 10
   xpaget prism file >>  $OUTFILE
   echo "" >> $OUTFILE
   xpaget prism tab -count >>  $OUTFILE
   echo "" >> $OUTFILE

   echo "" >> $OUTFILE
   echo " -- TEST LOAD FITS TABLE IN SAME EXTENSION " >>  $OUTFILE
   xpaset -p prism file $INDIR/acisf04425_000N003_evt1.fits >> $OUTFILE
   sleep 10
   xpaget prism file >>  $OUTFILE
   echo "" >> $OUTFILE
   xpaget prism tab -count >>  $OUTFILE
   echo "" >> $OUTFILE

   echo "" >> $OUTFILE
   echo " -- TEST LOAD FITS IMAGE IN NEW EXTENSION " >>  $OUTFILE
   xpaget prism file -newtab $INDIR/acisf162750752N001_1_bias0.fits >> $OUTFILE
   sleep 10
   xpaget prism file >>  $OUTFILE
   echo "" >> $OUTFILE
   xpaget prism tab -count >>  $OUTFILE
   echo "" >> $OUTFILE

   echo "" >> $OUTFILE
   echo " -- TEST SWITCH EXTENSION " >>  $OUTFILE
   
   xpaget prism tab -current >>  $OUTFILE
   echo "" >> $OUTFILE
   xpaget prism tab 0 >>  $OUTFILE
   xpaget prism file >>  $OUTFILE
   echo "" >> $OUTFILE
   xpaget prism tab 1 >>  $OUTFILE
   xpaget prism file >>  $OUTFILE
   echo "" >> $OUTFILE

   xpaget prism tab -remove
   xpaget prism tab -count >>  $OUTFILE
   echo "" >> $OUTFILE
   xpaget prism file >>  $OUTFILE
   echo "" >> $OUTFILE

else
   echo "FAILED: prism application time out at launch" >> $OUTFILE
fi
   xpaget prism quit

# remove variable portion of path from log file
sed -e "s;${INDIR};;g"  $OUTFILE > $OUTFILE2
#
# Run checks on the output files.
#
printf ". "
state=0
diff "$OUTFILE2" "$SAVEDIR/${TESTID}.txt" 2>&1 >> $OUTDIR/diff.log
if test $? != 0
then
   state=1
fi

#
# Exit condition
#
if test $state = 0
then
  echo "PASS"
  if echo "$OUTDIR" | grep "$TESTID" >/dev/null
  then
     rm $OUTFILE
     rm $OUTFILE2
     rm $OUTDIR/diff.log
     rmdir $OUTDIR
  fi
else
  echo "FAIL"
fi

exit $state
